/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.schemes;

import java.awt.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision$
 */
public class ResidueColourScheme implements ColourSchemeI
{
    Color[] colors;
    int threshold = 0;

    /* Set when threshold colouring to either pid_gaps or pid_nogaps*/
    protected String ignoreGaps = "pid_gaps";

    /** DOCUMENT ME!! */
    public Hashtable [] consensus;

    /**
     * Creates a new ResidueColourScheme object.
     *
     * @param colors DOCUMENT ME!
     * @param threshold DOCUMENT ME!
     */
    public ResidueColourScheme(Color[] colors, int threshold)
    {
        this.colors = colors;
        this.threshold = threshold;
    }

    /**
     * Creates a new ResidueColourScheme object.
     */
    public ResidueColourScheme()
    {
    }

    /**
     * DOCUMENT ME!
     *
     * @param consensus DOCUMENT ME!
     */
    public void setConsensus(Vector vconsensus)
    {
       int i, iSize=vconsensus.size();
       consensus = new Hashtable[iSize];
       for(i=0; i<iSize; i++)
        consensus[i] = (Hashtable)vconsensus.elementAt(i);
    }

    /**
     * DOCUMENT ME!
     *
     * @param aa DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Color findColour(String aa)
    {
        return colors[((Integer) (ResidueProperties.aaHash.get(aa))).intValue()];
    }

    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     * @param j DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Color findColour(String s, int j)
    {
        int index = ((Integer) (ResidueProperties.aaHash.get(s))).intValue();

        if ((threshold == 0) || aboveThreshold(ResidueProperties.aa[index], j))
        {
            return colors[index];
        }
        else
        {
            return Color.white;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getThreshold()
    {
        return threshold;
    }

    /**
     * DOCUMENT ME!
     *
     * @param ct DOCUMENT ME!
     */
    public void setThreshold(int ct, boolean ignoreGaps)
    {
        threshold = ct;
        if(ignoreGaps)
          this.ignoreGaps = "pid_nogaps";
        else
          this.ignoreGaps = "pid_gaps";
    }

    /**
     * DOCUMENT ME!
     *
     * @param s DOCUMENT ME!
     * @param j DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean aboveThreshold(String s, int j)
    {
        if ((((Integer) consensus[j].get("maxCount")).intValue() != -1) &&
                consensus[j].contains(s))
        {
            float ratio =  ((Float)consensus[j].get(ignoreGaps)).floatValue();

            if (ratio >= threshold)
            {
                return true;
            }
        }

        return false;
    }
}
